<?php

// autoload_psr4.php @generated by Composer

$vendorDir = dirname(__DIR__);
$baseDir = dirname($vendorDir);

return array(
    'Symfony\\Polyfill\\Php84\\' => array($vendorDir . '/symfony/polyfill-php84'),
    'Symfony\\Polyfill\\Php83\\' => array($vendorDir . '/symfony/polyfill-php83'),
    'Symfony\\Polyfill\\Ctype\\' => array($vendorDir . '/symfony/polyfill-ctype'),
    'Symfony\\Component\\Yaml\\' => array($vendorDir . '/symfony/yaml'),
    'Symfony\\Component\\TypeInfo\\' => array($vendorDir . '/symfony/type-info'),
    'Symfony\\Component\\Finder\\' => array($vendorDir . '/symfony/finder'),
    'Svg\\' => array($vendorDir . '/dompdf/php-svg-lib/src/Svg'),
    'Socket\\Raw\\' => array($vendorDir . '/clue/socket-raw/src'),
    'Sabberworm\\CSS\\' => array($vendorDir . '/sabberworm/php-css-parser/src'),
    'RingCentral\\Psr7\\' => array($vendorDir . '/ringcentral/psr7/src'),
    'Recurr\\' => array($vendorDir . '/simshaun/recurr/src/Recurr'),
    'React\\Stream\\' => array($vendorDir . '/react/stream/src'),
    'React\\Socket\\' => array($vendorDir . '/react/socket/src'),
    'React\\Promise\\Timer\\' => array($vendorDir . '/react/promise-timer/src'),
    'React\\Promise\\Stream\\' => array($vendorDir . '/react/promise-stream/src'),
    'React\\Promise\\' => array($vendorDir . '/react/promise/src'),
    'React\\Http\\' => array($vendorDir . '/react/http/src'),
    'React\\EventLoop\\' => array($vendorDir . '/react/event-loop/src'),
    'React\\Dns\\' => array($vendorDir . '/react/dns/src'),
    'React\\Datagram\\' => array($vendorDir . '/react/datagram/src'),
    'React\\ChildProcess\\' => array($vendorDir . '/react/child-process/src'),
    'React\\Cache\\' => array($vendorDir . '/react/cache/src'),
    'React\\Async\\' => array($vendorDir . '/react/async/src'),
    'Ramsey\\Uuid\\' => array($vendorDir . '/ramsey/uuid/src'),
    'Ramsey\\Collection\\' => array($vendorDir . '/ramsey/collection/src'),
    'Radebatz\\TypeInfoExtras\\' => array($vendorDir . '/radebatz/type-info-extras/src'),
    'Psr\\Log\\' => array($vendorDir . '/psr/log/src'),
    'Psr\\Http\\Server\\' => array($vendorDir . '/psr/http-server-middleware/src', $vendorDir . '/psr/http-server-handler/src'),
    'Psr\\Http\\Message\\' => array($vendorDir . '/psr/http-factory/src', $vendorDir . '/psr/http-message/src'),
    'Psr\\Http\\Client\\' => array($vendorDir . '/psr/http-client/src'),
    'Psr\\Container\\' => array($vendorDir . '/psr/container/src'),
    'Predis\\' => array($vendorDir . '/predis/predis/src'),
    'PhpParser\\' => array($vendorDir . '/nikic/php-parser/lib/PhpParser'),
    'PHPStan\\PhpDocParser\\' => array($vendorDir . '/phpstan/phpdoc-parser/src'),
    'OpenApi\\' => array($vendorDir . '/zircote/swagger-php/src'),
    'Masterminds\\' => array($vendorDir . '/masterminds/html5/src'),
    'Jfcherng\\Utility\\' => array($vendorDir . '/jfcherng/php-mb-string/src', $vendorDir . '/jfcherng/php-color-output/src'),
    'Jfcherng\\Diff\\' => array($vendorDir . '/jfcherng/php-diff/src', $vendorDir . '/jfcherng/php-sequence-matcher/src'),
    'GuzzleHttp\\Psr7\\' => array($vendorDir . '/guzzlehttp/psr7/src'),
    'GuzzleHttp\\Promise\\' => array($vendorDir . '/guzzlehttp/promises/src'),
    'GuzzleHttp\\' => array($vendorDir . '/guzzlehttp/guzzle/src'),
    'FontLib\\' => array($vendorDir . '/dompdf/php-font-lib/src/FontLib'),
    'Fig\\Http\\Message\\' => array($vendorDir . '/fig/http-message-util/src'),
    'Evenement\\' => array($vendorDir . '/evenement/evenement/src'),
    'Dompdf\\' => array($vendorDir . '/dompdf/dompdf/src'),
    'Doctrine\\Deprecations\\' => array($vendorDir . '/doctrine/deprecations/src'),
    'Doctrine\\Common\\Collections\\' => array($vendorDir . '/doctrine/collections/src'),
    'Cron\\' => array($vendorDir . '/dragonmantank/cron-expression/src/Cron'),
    'ConnectionManager\\Extra\\' => array($vendorDir . '/clue/connection-manager-extra/src'),
    'Clue\\Redis\\Protocol\\' => array($vendorDir . '/clue/redis-protocol/src'),
    'Clue\\React\\Utf8\\' => array($vendorDir . '/clue/utf8-react/src'),
    'Clue\\React\\Term\\' => array($vendorDir . '/clue/term-react/src'),
    'Clue\\React\\Stdio\\' => array($vendorDir . '/clue/stdio-react/src'),
    'Clue\\React\\Socks\\' => array($vendorDir . '/clue/socks-react/src'),
    'Clue\\React\\Soap\\' => array($vendorDir . '/clue/soap-react/src'),
    'Clue\\React\\Redis\\' => array($vendorDir . '/clue/redis-react/src'),
    'Clue\\React\\Mq\\' => array($vendorDir . '/clue/mq-react/src'),
    'Clue\\React\\HttpProxy\\' => array($vendorDir . '/clue/http-proxy-react/src'),
    'Brick\\Math\\' => array($vendorDir . '/brick/math/src'),
);
