<?php

declare(strict_types=1);

namespace Jfcherng\Diff\Renderer\Html;

/**
 * HTML Json diff generator.
 *
 * @deprecated 6.8.0 Use the "JsonHtml" renderer instead.
 */
final class Json extends JsonHtml
{
}
